'use strict';

var axios = require('axios');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var axios__default = /*#__PURE__*/_interopDefaultLegacy(axios);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

var T = {
    POSTGRESTER_DEPRECATION_BASE_URI: 'Warning: Using `baseUri` is now deprecated and will removed in v2.0.0: https://github.com/ivangabriele/postgrester/wiki/Deprecations#baseuri-option',
};

const Postgrester = class Postgrester {
    constructor({ axiosConfig, axiosInstance, baseUri }) {
        this.ands = [];
        this.foreignSorters = {};
        this.isAnd = false;
        this.isNot = false;
        this.isOr = false;
        this.limit = -1;
        this.offset = -1;
        this.ors = [];
        this.queries = [];
        this.selectors = [];
        this.sorters = [];
        if (baseUri !== undefined) {
            // eslint-disable-next-line no-console
            console.warn(T.POSTGRESTER_DEPRECATION_BASE_URI);
        }
        if (axiosInstance !== undefined) {
            this.axios = axiosInstance;
        }
        else {
            const newAxiosConfig = axiosConfig !== undefined ? axiosConfig : {};
            if (baseUri !== undefined) {
                newAxiosConfig.baseURL = baseUri;
            }
            this.axios = axios__default['default'].create(newAxiosConfig);
        }
    }
    get and() {
        this.isAnd = true;
        return this;
    }
    get or() {
        this.isOr = true;
        return this;
    }
    get not() {
        this.isNot = true;
        return this;
    }
    reset() {
        this.ands = [];
        this.foreignSorters = {};
        this.isAnd = false;
        this.isOr = false;
        this.isNot = false;
        this.limit = -1;
        this.offset = -1;
        this.ors = [];
        this.queries = [];
        this.selectors = [];
        this.sorters = [];
    }
    buildUri(path, isGet = false) {
        const params = [];
        // Selectors
        if (isGet) {
            params.push(`select=${this.selectors.length !== 0 ? this.selectors.join(',') : '*'}`);
        }
        // Filters
        this.queries.forEach(query => params.push(query));
        // "and" Filters
        if (this.ands.length !== 0) {
            const isNot = this.isNot ? 'not.' : '';
            params.push(`${isNot}and=(${this.ands.join(',')})`);
        }
        // "or" Filters
        if (this.ors.length !== 0) {
            const isNot = this.isNot ? 'not.' : '';
            params.push(`${isNot}or=(${this.ors.join(',')})`);
        }
        if (!isGet) {
            const query = params.join('&');
            return query.length > 0 ? `${path}?${params.join('&')}` : path;
        }
        // Sorters
        if (this.sorters.length !== 0) {
            params.push(`order=${this.sorters.join(',')}`);
        }
        // Foreign Sorters
        const foreignSorterPairs = Object.entries(this.foreignSorters);
        if (foreignSorterPairs.length !== 0) {
            foreignSorterPairs.forEach(([resource, sorters]) => {
                params.push(`${resource}.order=${sorters.join(',')}`);
            });
        }
        // Pagination
        if (this.limit !== -1) {
            params.push(`limit=${this.limit}`);
            params.push(`offset=${this.offset}`);
        }
        return `${path}?${params.join('&')}`;
    }
    get(path, withPagesLength = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const uri = this.buildUri(path, true);
            const { limit } = this;
            this.reset();
            const config = withPagesLength
                ? {
                    headers: {
                        Prefer: 'count=exact',
                    },
                }
                : {};
            const { data, headers } = yield this.axios.get(uri, config);
            let totalLength = -1;
            let pagesLength = -1;
            if (limit > 0 && withPagesLength && headers['content-range'] !== undefined) {
                totalLength = Number(headers['content-range'].split('/')[1]);
                pagesLength = Math.ceil(totalLength / limit);
            }
            return { data, pagesLength, totalLength };
        });
    }
    post(path, data, upsertOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            this.reset();
            // Upsert
            // http://postgrest.org/en/v7.0.0/api.html#upsert
            if (Array.isArray(data)) {
                if (upsertOptions.resolution === undefined) {
                    // eslint-disable-next-line no-param-reassign
                    upsertOptions.resolution = 'merge-duplicates';
                }
                const { onConflict, resolution } = upsertOptions;
                const fullPath = onConflict !== undefined ? `${path}?on_conflict=${onConflict}` : path;
                yield this.axios.post(fullPath, data, {
                    headers: {
                        Prefer: `resolution=${resolution}`,
                    },
                });
                return;
            }
            yield this.axios.post(path, data);
        });
    }
    patch(path, data) {
        return __awaiter(this, void 0, void 0, function* () {
            const uri = this.buildUri(path);
            this.reset();
            yield this.axios.patch(uri, data);
        });
    }
    put(path, data) {
        return __awaiter(this, void 0, void 0, function* () {
            const uri = this.buildUri(path);
            this.reset();
            yield this.axios.put(uri, data);
        });
    }
    delete(path) {
        return __awaiter(this, void 0, void 0, function* () {
            const uri = this.buildUri(path);
            this.reset();
            yield this.axios.delete(uri);
        });
    }
    select(selector) {
        this.selectors.push(selector);
        return this;
    }
    orderBy(column, isDesc = false) {
        if (column.includes('.')) {
            const [foreignResource, foreignColumn] = column.split('.');
            if (this.foreignSorters[foreignResource] === undefined) {
                this.foreignSorters[foreignResource] = [];
            }
            this.foreignSorters[foreignResource].push(`${foreignColumn}.${isDesc ? 'desc' : 'asc'}`);
        }
        else {
            this.sorters.push(`${column}.${isDesc ? 'desc' : 'asc'}`);
        }
        return this;
    }
    page(pageIndex, limit = 10) {
        this.limit = limit;
        this.offset = pageIndex * limit;
        return this;
    }
    eq(column, value, withQuotes = false) {
        if (typeof value === 'boolean' || value === null) {
            return this.is(column, value);
        }
        const mayBeNot = this.isNot ? 'not.' : '';
        const cleanValue = withQuotes ? `"${value}"` : value;
        if (this.isAnd) {
            this.ands.push(`${column}.eq.${cleanValue}`);
        }
        else if (this.isOr) {
            this.ors.push(`${column}.eq.${cleanValue}`);
        }
        else {
            this.queries.push(`${column}=${mayBeNot}eq.${cleanValue}`);
            this.isNot = false;
        }
        return this;
    }
    neq(column, value, withQuotes = false) {
        if (typeof value === 'boolean' || value === null) {
            return this.not.is(column, value);
        }
        const mayBeNot = this.isNot ? 'not.' : '';
        const cleanValue = withQuotes ? `"${value}"` : value;
        if (this.isAnd) {
            this.ands.push(`${column}.neq.${cleanValue}`);
        }
        else if (this.isOr) {
            this.ors.push(`${column}.neq.${cleanValue}`);
        }
        else {
            this.queries.push(`${column}=${mayBeNot}neq.${cleanValue}`);
            this.isNot = false;
        }
        return this;
    }
    gt(column, value, isInclusive = false) {
        const mayBeNot = this.isNot ? 'not.' : '';
        const operator = isInclusive ? 'gte' : 'gt';
        if (this.isAnd) {
            this.ands.push(`${column}.${operator}.${value}`);
        }
        else if (this.isOr) {
            this.ors.push(`${column}.${operator}.${value}`);
        }
        else {
            this.queries.push(`${column}=${mayBeNot}${operator}.${value}`);
            this.isNot = false;
        }
        return this;
    }
    gte(column, value) {
        this.gt(column, value, true);
        return this;
    }
    lt(column, value, isInclusive = false) {
        const mayBeNot = this.isNot ? 'not.' : '';
        const operator = isInclusive ? 'lte' : 'lt';
        if (this.isAnd) {
            this.ands.push(`${column}.${operator}.${value}`);
        }
        else if (this.isOr) {
            this.ors.push(`${column}.${operator}.${value}`);
        }
        else {
            this.queries.push(`${column}=${mayBeNot}${operator}.${value}`);
            this.isNot = false;
        }
        return this;
    }
    lte(column, value) {
        this.lt(column, value, true);
        return this;
    }
    like(column, value) {
        const mayBeNot = this.isNot ? 'not.' : '';
        if (this.isAnd) {
            this.ands.push(`${column}.like.*${value}*`);
        }
        else if (this.isOr) {
            this.ors.push(`${column}.like.*${value}*`);
        }
        else {
            this.queries.push(`${column}=${mayBeNot}like.*${value}*`);
            this.isNot = false;
        }
        return this;
    }
    ilike(column, value) {
        const mayBeNot = this.isNot ? 'not.' : '';
        if (this.isAnd) {
            this.ands.push(`${column}.ilike.*${value}*`);
        }
        else if (this.isOr) {
            this.ors.push(`${column}.ilike.*${value}*`);
        }
        else {
            this.queries.push(`${column}=${mayBeNot}ilike.*${value}*`);
            this.isNot = false;
        }
        return this;
    }
    in(column, values, withQuotes = false) {
        const finalValues = withQuotes ? values.map(value => `"${value}"`) : values;
        const mayBeNot = this.isNot ? 'not.' : '';
        if (this.isAnd) {
            this.ands.push(`${column}.in.(${finalValues.join(',')})`);
        }
        else if (this.isOr) {
            this.ors.push(`${column}.in.(${finalValues.join(',')})`);
        }
        else {
            this.queries.push(`${column}=${mayBeNot}in.(${finalValues.join(',')})`);
            this.isNot = false;
        }
        return this;
    }
    is(column, value) {
        const mayBeNot = this.isNot ? 'not.' : '';
        if (this.isAnd) {
            this.ands.push(`${column}.is.${String(value)}`);
        }
        else if (this.isOr) {
            this.ors.push(`${column}.is.${String(value)}`);
        }
        else {
            this.queries.push(`${column}=${mayBeNot}is.${String(value)}`);
        }
        this.isNot = false;
        return this;
    }
};
// Allow use of default import syntax in TypeScript:
module.exports.default = Postgrester;

// tslint:disable-next-line: import-name
/**
 * Create an instance of Postgrester.
 */
function createInstance(config) {
    return new Postgrester(config);
}
// Create the default instance to be exported:
const postgrester = createInstance({});
postgrester.Postgrester = Postgrester;
// Factory for creating new instances:
postgrester.create = function create(options) {
    return createInstance(options);
};
// Allow use of default import syntax in TypeScript:
module.exports.default = postgrester;

module.exports = postgrester;
